# -*- coding: ISO-8859-1 -*-
# Copyright 2004 Fundacion Via Libre
#
# This file is part of PAPO.
# 
# PAPO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# PAPO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PAPO; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Model import DIAEditingContext

from Generic.Responder import Responder
from cimarron import getEngine, Failed, Accept
# get the engine
ui= getEngine ()

class Controller (Responder):
    """
    """
    __kwargs = ('defaultValue', 'value', 'getter', 'action')
    __doc__ += "%s\n" % (__kwargs,)
    _default_value = None
    __obligs = {'_action': lambda: None,
                '_getter': lambda: None,
                '_value': lambda: None,
                }

    def setAction(self, action):
        """
        The action is a callback that is called with the Control as
        sole argument when the user activates the Control and the
        delegates agree it would be a good idea.
        """
        self._action = action
        return True
    def doAction(self, *args, **kwargs):
        """
        Call the action.

        If the action is callable, call it, and return whatever it
        returns. If the action is *not* callable, return the action
        itself.
        """
        if callable(self._action):
            return self._action(self, *args, **kwargs)
        else:
            return self._action

    def setGetter(self, getter):
        """
        The getter is a callback that is called when the Control wants
        the value from the Model. It should return that value, or
        fail.
        """
        self._getter = getter
    def getGetter(self):
        return self._getter
    def callGetter(self, *args, **kwargs):
        """
        Call the getter.

        If the getter is callable, call it, and return whatever it
        returns. If the getter is *not* callable, return the getter
        itself.
        """
        if callable(self._getter):
            return self._getter(self, *args, **kwargs)
        else:
            return self._getter

class ConsistentController (Controller):
    """
    Controller that warns the user that is trying to close the window
    when the data is dirty.
    """
    __kwargs= ('windowTitle', 'parent', 'model', 'modelClass')
    def __init__ (self, *args, **kwargs):
        # already takes care of model and modelClass params via setModel{,Class}()
        self.model= DIAEditingContext ()
        self._processArgs(ConsistentController, kwargs)

    def buildUI (self):
        self.win= ui.Window (parent=self.getParent(), title=self.getWindowTitle ())
        self.win.addDelegation (self)

        v= ui.VBox (parent=self.win)
        self.tabbook= ui.Notebook (parent=v)

        # the save button is connected to the action thru save()
        h= ui.HBox (parent=v)
        okButton= ui.Button (parent=h, action=lambda *ignore: self.save (), label=u'_Guardar')
        resetButton= ui.Button (parent=h, action=lambda *ignore: self.reset (), label=u'_Reestablecer valores anteriores')
    def show (self):
        self.win.show ()

    def setModelClass (self, modelClass):
        self.model.setModelClass (modelClass)
    def getModelClass (self):
        return self.model.getModelClass ()

    def setModel (self, model):
        self.model.setModel (model)
    def getModel (self):
        return self.model.getModel ()
    def updateModel (self, model=None):
        self.model.updateModel (model)
        # this could be conected to the Notebook's page change event
        self.update ()

    def setParent (self, parent):
        self.parent= parent
    def getParent (self):
        return self.parent

    def setWindowTitle (self, windowTitle):
        self.windowTitle= windowTitle
    def getWindowTitle (self):
        return self.windowTitle

    def save (self):        
        self.model.save ()

    def reset (self):
        self.model.reset ()
        self.update ()

    def update (self):
        pass

    def will_delete (self, *ignore):
        """
        delegate for the window destroy action.
        doesn't allow destroying it, just hides it
        """
        if self.model.isDirty ():
            # tis dirty, ask the user
            if ui.WarningDialog (message=u'Los cambios no han sido guardados. Qu desea hacer con ellos?', yea=u'_Guardar', nay= u'_Descartar').run ():
                self.save ()
            else:
                self.reset ()
        self.win.hide ()
        return Failed
