# -*- coding: ISO-8859-1 -*-
# Copyright 2004 Fundacion Via Libre
#
# This file is part of PAPO.
# 
# PAPO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# PAPO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PAPO; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# This module is just a Cimarron+Modeling example

# It's composed of three Controllers:

# Main controls the window, and fixes the hooks of the other two.
# It also carries the model
# wich can be searched with the first page
# and edited w/ the second.

# SearchPage will hold a search thing,
# but now it can look for a Country in the db
# or return an empty one if none found
# it has an action, which will call with the Country as only param

# DataPage holds a simple editing widget w/ save and reset buttons
# reset does nothing, but should rollback any changes not saved so far.
# save calls its action without any params

# The idea is that nor Search nor Data Page should know about Main's API,
# but this last one knows about the former two.
# Note this is the same with any other Cimarron's object.

# We perform the rollback cheating with EditingContexts.
# The idea is the following:
# We keep a child EditingContext wich holds the Country being edited.
# If the user hits the Reset button, we throw away the child EC, create a new one
# and ask for the Country again, using faultForGlobalID()

from whrandom import uniform

from Modeling.EditingContext import EditingContext
from PAPOv2.Country import Country

from Controllers.Controller import Controller, ConsistentController

from cimarron import getEngine
# get the engine
ui= getEngine ()

def random (m, n=None):
    """
    returns an uniformly generated integer in [m,n)
    or in [0,m) if n is not supplied.
    """
    if not n:
        return int(uniform (0, m))
    else:
        return int(uniform (m, n))

class SearchPage (Controller):
    """
    """
    def __init__ (self, *args, **kwargs):
        self._processArgs(SearchPage, kwargs)
        self.buildUI ()
    
    def buildUI (self):
        layout= ui.VBox ()
        
        # hitting the button calls the action with any Country() found.
        ui.Button (parent=layout, action=lambda *ignore: self.doAction (self.getCountry ()), label='_Elegir al azar!')

        # hitting this other one builds a new Country.
        ui.Button (parent=layout, action=lambda *ignore: self.doAction (Country ()), label='_Nuevo')
        self.ui= layout

    def getCountry (self):
        # fake for now
        countries= self.callGetter ().fetch ('Country')
        if countries:
            # get a random one
            country= countries[random (len (countries))]
        else:
            country= Country ()
        return country

class DataPage (Controller):
    """
    No special parameters, except for those inherited from Controller.
    """
    def __init__ (self, *args, **kwargs):
        super (DataPage, self).__init__ (*args, **kwargs)
        self.buildUI ()

    def buildUI (self):
        layout= ui.VBox ()

        # an editor for the name
        # the getter is connected to the Page's getter.
        # the setter too.
        h= ui.HBox (parent=layout)
        nameLabel= ui.Label (parent=h, label=u'_Nombre: ', useUnderline=True)
        self.nameInput= ui.TextEntry (
            parent=h,
            getter= lambda *ignore: self.callGetter().getName (),
            action= lambda control: self.callGetter().setName (control.getValue ()),
        )
        nameLabel.setMnemonicWidget (self.nameInput)
        
        self.ui= layout

    def update (self):
        # There must be a better way to do this...
        # basically we need to update all the controls.
        for i in [self.nameInput]:
            i.update ()

class Main (ConsistentController):
    """
    Defines the window with the notebook,
    instantiates a SearchPage and a DataPage
    links all the hooks.
    """
    __obligs= {'model': lambda: Country ()}

    def __init__ (self, *args, **kwargs):
        kwargs.update ({
            'windowTitle': u'ABM de pas',
            'model': Country (),
            'modelClass': Country,            
            })

        self._processArgs(Main, kwargs)

        self.buildUI ()

    def buildUI (self):
        super (Main, self).buildUI ()
        
        # build the search page. the action is to update our model
        self.firstPage= SearchPage (getter=lambda *ignore: self.model, action=lambda control, data, *ignore: self.changeModel (data))
        self.tabbook.appendPage (self.firstPage.ui, u'Busqueda')

        # build the editing page. the getter reaches our model ans the action reaches our save() method.
        self.secondPage= DataPage (getter=lambda *ignore: self.getModel())
        self.tabbook.appendPage (self.secondPage.ui, u'Datos')

    def setApp (self, app):
        self.app= app
    def getApp (self):
        return self.app

    def changeModel (self, model=None):
        """
        Change the model to the one selected and
        activate the second page
        (this is wanted behaviour, not something needed).
        """
        self.updateModel (model)
        self.tabbook.changePage (1)

    def save (self):
        """
        Save the data in the model and
        activate the first page
        (again, wanted behaviour)
        """
        super (Main, self).save ()
        self.tabbook.changePage (0)

    def reset (self):
        """
        just add the update of the data page
        """
        super (Main, self).reset ()
        self.secondPage.update ()

    def update (self):
        """
        just update the data page
        """
        self.secondPage.update ()

    def show (self):
        self.update ()
        super (Main, self).show ()
