# -*- python -*- coding: ISO-8859-1 -*-
# Copyright 2004 Fundacion Via Libre
#
# This file is part of PAPO.
# 
# PAPO is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# PAPO is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PAPO; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Modeling.EditingContext import EditingContext

class DIAEditingContext (object):
    """
    This class abstracts the trick done to achieve rollbackable EditingContexts.
    Holds a model, which can be edited.
    """
    __kwargs= ('model', 'modelClass')
    def __init__ (self):
        self.ec= EditingContext ()
        self.childEc= None
    
        self.updateCtxt ()

    def setModelClass (self, modelClass):
        self.modelClass= modelClass
    def getModelClass (self):
        return self.modelClass

    def setModel (self, model):
        self.model= model
    def getModel (self):
        return self.model
    def updateModel (self, model=None):
        """
        This is called when the user has selected some model.
        """
        if not model:
            model= self.getModel ()
        # this may seem redundant, but the reason for the calling to
        # faultForGlobalId() is to populate the childEc with the selected model
        # and returns the new model that is used to hold the temporary data
        # self.setModel (self.childEc.faultForGlobalID (model.globalID (), self.childEc))
        self.reset (model)

    def updateCtxt (self):
        """
        Throws away the child context if it exists
        and creates a new one.
        """
        if self.childEc:
            # throw away the child ec
            del self.childEc
        # create a new one
        self.childEc= EditingContext (self.ec)

    def fetch (self, *args, **kwargs):
        return self.ec.fetch (*args, **kwargs)

    def save (self):
        """
        Saves the changes to the model.
        """
        # propagate the changes down the line
        self.childEc.saveChanges ()
        self.ec.saveChanges ()

    def reset (self, model=None):
        """
        Restores the model to the previous state or
        sets the model to the param, throwing away any
        change to the actual model.
        """
        # dirty trick to acomplish rollback:
        try:
            # save the gid
            if not model:
                gid= self.getModel ().globalID ()
            else:
                gid= model.globalID ()
            # create a new child context
            self.updateCtxt ()
            # populate the new EC with the model
            self.setModel (self.childEc.faultForGlobalID (gid, self.childEc))
        except (TypeError, AttributeError):
            # faultForGlobalID failed; means data was not yet in the ec.
            self.setModel (self.getModelClass ()())

    def isDirty (self):
        return self.childEc.hasChanges ()
